(use util.combinations)

(define (printf . args)
  (display (apply format args)))

(define release-date "2007/12/18")

(define family "pad")

(define (weight-in-otf weight)
  (case weight
    ((regular) "Regular")
    ((semibold) "Semibold")
    ((bold) "Bold")
    (else "")))

(define (variant-in-otf variant)
  (case variant
    ((italic) "Italic")
    (else "")))

(define (variant-option variant)
  (case variant
    ((smallcaps) " -fsmcp")
	(else "")))

(define (figure-option figure)
  (case figure
    ((oldstyle) " -fonum")
	(else "")))

(define (weight-in-berry weight)
  (case weight
    ((regular) "r")
    ((semibold) "s")
    ((bold) "b")
    (else "")))

(define (variant-in-berry variant)
  (case variant
    ((italic) "i")
	((smallcaps) "c")
    (else "")))

(define (figure-in-berry figure)
  (case figure
    ((oldstyle) "j")
	(else "")))

(define (encoding-in-berry encoding)
  (case encoding
    ((t1) "8t")
    ((ts1) "8c")
    (else "")))

(define (otf-file weight variant)
  (string-append
    "AGaramondPro-"
    (if (and (eq? weight 'regular) (eq? variant 'italic))
      ""
	  (weight-in-otf weight))
	(variant-in-otf variant)
    ".otf"))

(define (berry-name family weight variant figure encoding)
  (string-append family
	             (weight-in-berry weight)
				 (variant-in-berry variant)
				 (figure-in-berry figure)
				 (encoding-in-berry encoding)))

(define (generate-one-font weight variant figure encoding)
  (define (enc-file)
    (cond
	  ((and (eq? weight 'bold) (eq? encoding 'ts1))
	    (string-append family
		               (weight-in-berry weight)
					   (encoding-in-berry encoding)
					   ".enc"))
	  ((and (eq? variant 'smallcaps) (eq? encoding 't1))
	    (string-append family
		               (variant-in-berry variant)
					   (encoding-in-berry encoding)
					   ".enc"))
	  (else
	    (string-append family
	                   (encoding-in-berry encoding)
		    		   ".enc"))))
  (printf "  echo generating ~a\n"
          (berry-name family weight variant figure encoding))
  (printf "  otftotfm -e ~a -fkern~a~a -n ~a --map-file=~a.map --no-updmap ~a\n"
          (enc-file)
		  (variant-option variant)
		  (figure-option figure)
		  (berry-name family weight variant figure encoding)
		  family
		  (otf-file weight variant)))

(define (generate-fonts)
  (define (wrapper weight variant)
	(when (eq? variant 'normal)
	  (printf "\nif [ -f ~a ]; then\n" (otf-file weight variant)))
	(when (eq? variant 'italic)
	  (printf "\nif [ -f ~a ]; then\n" (otf-file weight variant)))
    (unless (and (eq? weight 'bold) (eq? variant 'smallcaps))
      (for-each (lambda (p)
	              (apply generate-one-font
				         (cons weight (cons variant p))))
		        (cartesian-product '((lining oldstyle) (t1 ts1)))))
    (when (or (eq? variant 'smallcaps) (eq? variant 'italic))
      (printf "fi\n")))
  (printf "#!/bin/sh\n\nif [ -f ~a.map ]; then\n  rm ~a.map\nfi\n"
          family family)
  (for-each (cut apply wrapper <>)
            (cartesian-product '((regular semibold bold)
			                     (normal smallcaps italic)))))

(define (family-in-nsff family figure)
  (case figure
    ((oldstyle) (string-append family "j"))
	(else family)))

(define (encoding-in-nsff encoding)
  (case encoding
    ((t1) "T1")
	((ts1) "TS1")
	(else "")))

(define (weight-in-nsff weight)
  (case weight
    ((regular) "m")
	((semibold) "sb")
	((bold) "b")
	((extrabold) "eb")
	((bold-extended) "bx")
	(else "")))

(define (variant-in-nsff variant)
  (case variant
    ((normal) "n")
	((smallcaps) "sc")
	((italic) "it")
	(else "")))

(define (generate-fd-files)
  (define (declare-variants indent issubstitute nosmallcaps encoding family figure weight font-weight)
    (define (declare-variant variant)
	  (when (> indent 0)
	    (display "  "))
	  (printf "\\DeclareFontShape{~a}{~a}{~a}{~a}{<-> ~a}{}\n"
	          (encoding-in-nsff encoding)
			  (family-in-nsff family figure)
			  (weight-in-nsff weight)
			  (variant-in-nsff variant)
			  (if (or issubstitute
			          (and nosmallcaps (eq? variant 'smallcaps)))
			    (format "ssub * ~a/~a/~a"
				        (family-in-nsff family figure)
						(weight-in-nsff (if issubstitute font-weight weight))
						(variant-in-nsff (if issubstitute variant 'normal)))
			    (format "\\pad@@scale ~a" (berry-name family font-weight variant figure encoding)))))
    (for-each (cut declare-variant <>) '(normal smallcaps italic)))
  (define (declare-family encoding figure)
    (printf "\\ProvidesFile{~a~a.fd}[~a kkondo]\n\n"
	        encoding (family-in-nsff family figure) release-date)
    (printf "\\DeclareFontFamily{~a}{~a}{}\n"
	        (encoding-in-nsff encoding)
			(family-in-nsff family figure))
    (declare-variants 0 #f #f encoding family figure 'regular 'regular)
    (declare-variants 0 #f #f encoding family figure 'semibold 'semibold)
	(display "\\if@semibold\n")
    (declare-variants 1 #t #f encoding family figure 'bold 'semibold)
    (declare-variants 1 #f #t encoding family figure 'extrabold 'bold)
	(display "\\else\n")
    (declare-variants 1 #f #t encoding family figure 'bold 'bold)
    (declare-variants 1 #t #f encoding family figure 'extrabold 'bold)
	(display "\\fi\n")
    (declare-variants 0 #t #f encoding family figure 'bold-extended 'bold)
	(display "\n\\endinput\n"))
  (define (generate-fd encoding figure)
    (let ((out (open-output-file (format "~a~a.fd"
	                                     encoding
						                 (family-in-nsff family figure)))))
      (with-output-to-port out (lambda () (declare-family encoding figure)))))
  (for-each (cut apply generate-fd <>)
            (cartesian-product-right '((t1 ts1) (lining oldstyle)))))

(define (main args)
  (let ((out (open-output-file "genfonts")))
    (with-output-to-port out generate-fonts))
  (generate-fd-files))
